/*
 * Copyright 2005, 2006 University of Leiden.
 *
 * This file is part of MIA+EWS.
 *
 * MIA+EWS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MIA+EWS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MIA; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*******************************************************************************
* MPIA, NEVEC - MIDI project
*
* "@(#) $Id: table_row.c,v 1.5 2002/07/03 11:58:29 jdejong Exp $"
*
* who        when      what
* ---------  --------  ----------------------------------------------
* jdejong    2002-05-08 imported
*
*/

/* $Id: table_row.c,v 1.5 2002/07/03 11:58:29 jdejong Exp $ */
#define _POSIX_SOURCE_1
#include "vltPort.h"

/* $Id: table_row.c,v 1.5 2002/07/03 11:58:29 jdejong Exp $ */
/*----------------------------------------------------------------------- */
/* table row FITS binary tables routines for the VLTI interferometer      */
/*----------------------------------------------------------------------- */
#include <malloc.h>
#include <string.h>
#include "VLTItable.h"

table_row* make_table_row(long  trow, int ncol)
/*----------------------------------------------------------------------- */
/*   Make a table_row object                                              */
/*   Inputs:                                                              */
/*      trow       Row number number                                      */
/*      ncol       Number of columns                                      */
/*   Returns:                                                             */
/*      New table_row structure; NULL -> failed                           */
/*----------------------------------------------------------------------- */
{
  int i;

  table_row* out = (table_row*) malloc(sizeof(table_row));
  if (out==NULL) return out;  /* check allocation */

  out->trow = trow;
  out->ncol = ncol;
  out->data = (table_col**)malloc(ncol * sizeof(table_col*));
  for (i=0; i<ncol; i++) /* make blank column entries */
    out->data[i] = (table_col*)make_table_col(NULL);
  return out;
} /* end  make_table_row*/


void kill_table_row(table_row* row)
/*----------------------------------------------------------------------- */
/*   Destroy a table_row object                                           */
/*   Inputs:                                                              */
/*      table_row  Object to destroy                                      */
/*----------------------------------------------------------------------- */
{
  int i;
  if (row==NULL) return; /* validity check */
  /* free column objects */
  for (i=0; i<row->ncol; i++)  kill_table_col(row->data[i]);
  free (row->data);      /* finish column storage */
  free (row);            /* free storage */
} /* end kill_table_row*/

void table_write_row (fitsfile *fptr, table_def *tabledef, 
		      table_row *row_data, int *ierr)
/*----------------------------------------------------------------------- */
/*   Function to write a row in the currently open table                  */
/*   Inputs:                                                              */
/*      fptr       Pointer to open FITSIO file structure                  */
/*      tabledef   Pointer to the table defintion structure               */
/*      row_data   Row data structure                                     */
/*   Output:                                                              */
/*      ierr      Error code: 0 => ok, returns if initially positive      */
/*----------------------------------------------------------------------- */
{
  int i, ncol;
  table_col* col_data;
  char* grumble[1];

  /* Are we already in an error state? */
  if (*ierr>0) return;
  
  /* check for bad pointers */
  if ((fptr==NULL) || (row_data==NULL)) {
    *ierr=10000+1;
    return;}

  /* loop over columns */
  ncol = row_data->ncol;
  for (i=0; i<ncol; i++) {
    if (tabledef->colcount[i]>0) { /* only if actual data */
      col_data = row_data->data[i];
      /* have to do something special for Strings */
      /* cfitsio does something different with strings */
      if (tabledef->coldatatype[i]==TSTRING) {
	grumble[0] = col_data->tdata.tchar;
	fits_write_col_str(fptr, i+1, 
			   row_data->trow, 1, 1,  
			   grumble,
			   ierr);
      } else {
	fits_write_col(fptr, tabledef->coldatatype[i], i+1,
		       row_data->trow, 1, tabledef->colcount[i],
		       col_data->tdata.tvoid,
		       ierr);
      } /* end of special treatment of strings */
    } /* end of positive element count */
  }
} /* end table_write_row */


void table_read_row (fitsfile *fptr, table_def *tabledef, 
		     table_row *row_data, int *ierr)
/*----------------------------------------------------------------------- */
/*   Function to read a row in the currently open table                   */
/*   Inputs:                                                              */
/*      fptr       Pointer to open FITSIO file structure                  */
/*      tabledef   Pointer to the table defintion structure               */
/*      row_data   Row data structure                                     */
/*   Output:                                                              */
/*      row_data   Row data structure filled in                           */
/*      ierr      Error code: 0 => ok, returns if initially positive      */
/*----------------------------------------------------------------------- */
{
  int i, ncol, anynul;
  table_col* col_data;
  char *grumble[1], nulstr[16];

  /* Are we already in an error state? */
  if (*ierr>0) return;
  
  /* check for bad pointers */
  if ((fptr==NULL) || (row_data==NULL)) {
    *ierr=10000+1;
    return;}

  /* loop over columns */
  for (i=0;i<16;i++) nulstr[i] = 0; /* no null data checking */
  ncol = row_data->ncol;
  for (i=0; i<ncol; i++) {
    if (tabledef->colcount[i]>0) { /* only if actual data */
      col_data = row_data->data[i];
      /* have to do something special for Strings */
      /* cfitsio does something different with strings */
      if (tabledef->coldatatype[i]==TSTRING) {
	grumble[0] = col_data->tdata.tchar;
	fits_read_col_str(fptr,  i+1, row_data->trow, 1, 1,  
			  nulstr, grumble, &anynul,
			  ierr);
      } else {
        /* this prevents an overflow due to the TZERO setting */
        fits_set_tscale (fptr,i+1,1,0,ierr);
        fits_read_col(fptr,tabledef->coldatatype[i], i+1, 
                      row_data->trow, 1, tabledef->colcount[i],
                      nulstr, col_data->tdata.tvoid,
                      &anynul, ierr);
      } /* end of special treatment of strings */
    } /* end of positive element count */
  }
} /* end table_read_row */


/* */
